/***************************************************************************/
/** @file     main.cpp
  * @brief    The main initialization function.
  *
  * @b begin      : Fre Okt 22 17:39:47 CEST 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
/** @mainpage
  *
  * <p>%Ksetiwatch is a monitoring tool for the
  * <a href="http://setiathome.ssl.berkeley.edu">%Seti@home</a> distributed
  * computing project, which searches for signals of extraterrestrial life.
  * It displays the state of the %Seti@home client(s) running on your
  * computer or in your local network, and logs/manages completed work units
  * and interesting signals. In addition, %Ksetiwatch features a skymap that
  * displays all work units processed, a graphical display of scientific data
  * (gaussians, pulses, and triplets), a dock icon with detailed status
  * information about a client, the option to start/stop the %SETI@home
  * clients, and much more.</p>
  **************************************************************************/
#include <kapp.h>
#include <kcmdlineargs.h>

#include "ksetiwatch.h"
#include "../config.h"

/** The program starts here. */
int main(int argc, char* argv[])
{
KCmdLineArgs::init(argc, argv,
                   "ksetiwatch",
                   "Ksetiwatch",
                   "A SETI@home monitoring tool",
                   VERSION);

KsetiwatchApp a;

Ksetiwatch* ksetiwatch = new Ksetiwatch("ksetiwatch");
a.setMainWidget(ksetiwatch);

if(globalopts->MinimizeToPanel == false)
  {
  if( ksetiwatch->startMinimized() )
    ksetiwatch->showMinimized();
  else
    ksetiwatch->show();
  }
else if( ksetiwatch->startMinimized() == false )
  ksetiwatch->show();

return a.exec();
}








