/***************************************************************************/
/** @file     options.cpp
  * @brief    The Options tab page (implementation).
  *
  * @b begin      : Tue Nov 23 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <kapp.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kaudioplayer.h>
#include <kiconloader.h>

#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlineedit.h>

#include "ksetiwatch.h"
#include "options.h"
#include "setiloc.h"

/*------------------------------------------------------------------------ */
Options::Options(QWidget *parent, const char *name )
       : OptionsTmpl(parent,name)
{
slotSetRefreshInterval(globalopts->RefreshInterval);
RefreshIntSlider->setValue(globalopts->RefreshInterval);

MinToPanelChkBox->setChecked(globalopts->MinimizeToPanel);
DrawGridChkBox->setChecked(globalopts->DrawGrid);

HMSRadioBtn->setChecked(globalopts->hms);
DHMSRadioBtn->setChecked(!globalopts->hms);

SpikeChkBox->setChecked(globalopts->ReportSpike);
GaussianChkBox->setChecked(globalopts->ReportGaussian);
PulseChkBox->setChecked(globalopts->ReportPulse);
TripletChkBox->setChecked(globalopts->ReportTriplet);
ShowRecordChkBox->setChecked(globalopts->ShowRecords);

PlayBtn->setPixmap(SmallIcon("1rightarrow"));
SoundBtn->setPixmap(SmallIcon("fileopen"));
SoundChkBox->setChecked(globalopts->PlaySound);
SoundLineEdit->setText(globalopts->HighSignalSound);
}

/*------------------------------------------------------------------------ */
Options::~Options()
{
}

/*------------------------------------------------------------------------ */
void Options::slotSetTimeFormat(int id)
{
if(id == 1)
  globalopts->hms = true;
else
  globalopts->hms = false;
}

/*------------------------------------------------------------------------ */
void Options::slotSetReportSignals(int id)
{
switch(id)
  {
  case 0: globalopts->ReportSpike = SpikeChkBox->isChecked();
          break;
  case 1: globalopts->ReportGaussian = GaussianChkBox->isChecked();
          break;
  case 2: globalopts->ReportPulse = PulseChkBox->isChecked();
          break;
  case 3: globalopts->ReportTriplet = TripletChkBox->isChecked();
          break;
  case 4: globalopts->ShowRecords = ShowRecordChkBox->isChecked();
          break;
  default:
          break;
  }
}

/*------------------------------------------------------------------------ */
void Options::slotSetMinToPanel()
{
globalopts->MinimizeToPanel = MinToPanelChkBox->isChecked();
emit dockStatusChanged();
}

/*------------------------------------------------------------------------ */
void Options::slotSetDrawGrid()
{
globalopts->DrawGrid = DrawGridChkBox->isChecked();
}

/*------------------------------------------------------------------------ */
void Options::slotSetSoundOption()
{
globalopts->PlaySound = SoundChkBox->isChecked();
}

/*------------------------------------------------------------------------ */
void Options::slotSelectSound()
{
QString f = KFileDialog::getOpenFileName();
if(!f.isEmpty())
  {
  setSound(f);
  SoundLineEdit->setText(f);
  }
}

/*------------------------------------------------------------------------ */
void Options::slotPlaySound()
{
if(!globalopts->HighSignalSound.isEmpty())
  KAudioPlayer::play(globalopts->HighSignalSound);
}

/*------------------------------------------------------------------------ */
void Options::slotSetSound()
{
setSound(SoundLineEdit->text());
}

/*------------------------------------------------------------------------ */
void Options::setSound(const QString& s)
{
globalopts->HighSignalSound = s;
if(globalopts->HighSignalSound.isEmpty())
  PlayBtn->setEnabled(false);
else
  PlayBtn->setEnabled(true);
}

/*------------------------------------------------------------------------ */
void Options::slotSetRefreshInterval(int ri)
{
QString title, buf;
int m, s;

if(ri != globalopts->RefreshInterval)
  {
  for(SetiLoc* loc=Ksetiwatch::locationList().first(); loc != 0;
      loc=Ksetiwatch::locationList().next())
    {
    loc->setRefreshInterval(ri);
    }
  }
  globalopts->RefreshInterval = ri;
  title = i18n("Refresh Interval: ");
  m = (int)(ri/60);
  s = ri % 60;
  if(m)
    {
    buf.sprintf("%d min ", m);
    title += buf;
    }
  if(s)
    {
    if(m)
      buf.sprintf("%02d s", s);
    else
      buf.sprintf("%d s", s);
    title += buf;
    }
  RefreshIntSliderBox->setTitle(title);	
}

/*------------------------------------------------------------------------ */
void Options::slotEnableRecordCheckbox(bool)
{
if(SpikeChkBox->isChecked() || GaussianChkBox->isChecked() ||
   PulseChkBox->isChecked() || TripletChkBox->isChecked())
  {
  ShowRecordChkBox->setEnabled(true);
  }
else
  {
  ShowRecordChkBox->setEnabled(false);
  }
}

#include "options.moc"

