/***************************************************************************/
/** @file     options.h
  * @brief    The Options tab page (declaration).
  *
  * @b begin      : Tue Nov 23 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef OPTIONS_H
#define OPTIONS_H

#include "optionstmpl.h"

/***************************************************************************/
/** @file     optionstmpl.h
  * @brief    The base class for the Options tab page (declaration).
  *
  * Layout of the Options tab page, automatically generated from the Qt
  * Designer user interface (ui) file @c optionstmpl.ui. Any manual
  * modifications in this file will be overwritten.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  */

/***************************************************************************/
/** @file     optionstmpl.cpp
  * @brief    The base class for the Options tab page (implementation).
  *
  * Layout of the Options tab page, automatically generated from the Qt
  * Designer user interface (ui) file @c optionstmpl.ui. Any manual
  * modifications in this file will be overwritten.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  */

/***************************************************************************/
/** @class OptionsTmpl
  * @brief The base class for the Options widget.
  *
  * This class is automatically generated from a Qt Designer user interface
  * (ui) file. Any manual changes to it will be overwritten.
  */

/** The widget for the Options tab page.
  */
class Options : public OptionsTmpl
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  Options(QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~Options();

protected: // methods
  /** Stores the path to the sound file in the %application-wide options.
    *
    * Only if the string @a s is non-empty, the @e Play button will be
    * enabled.
    *
    * @param s A string containing the path to the sound file (which must be
    *          in @c wav format).
    */
  void setSound(const QString& s);

protected slots:
  /** Stores the settings of @e Time @e Format radio buttons in the
    * %application-wide options.
    *
    * @param id The ID number of the selected radio button. Either 0
    *           for hh:mm:ss, or 1 for dd:hh:mm:ss.
    */
  void slotSetTimeFormat(int id);
  /** Stores the settings of @e Report @e New @e Signals check boxes in the
    * %application-wide options.
    *
    * @param id The ID number of the clicked check box. Ranges from 0 to 4.
    */
  void slotSetReportSignals(int id);
  /** Stores the state of the @e Minimize @e To @e Panel check box in the
    * %application-wide options.
    */
  void slotSetMinToPanel();
  /** Stores the state of the @e Draw @e Grid check box in the
    * %application-wide options.
    */
  void slotSetDrawGrid();
  /** Sets the refresh interval of all %S@h locations to the value adjusted
    * with the slider.
    *
    * The refresh interval is also stored in the %application-wide options.
    *
    * @param ri The slider position representing the refresh interval in
    *           seconds.
    */
  void slotSetRefreshInterval(int ri);
  /** Stores the state of the @e Play @e Sound check box in the
    * %application-wide options.
    */
  void slotSetSoundOption();
  /** Opens a file dialog for the selection of a sound file.
    *
    * The selected sound file and the path will be passed to the corresponding
    * line edit field.
    */
  void slotSelectSound();
  /** Plays the currently selected sound. */
  void slotPlaySound();
  /** Sets the sound entered into the line edit field as the active sound. */
  void slotSetSound();
  /** Enables/disables the 'Show Records' check box.
    *
    * The 'Show Records' check box is enabled if at least one of the four
    * different signal check boxes is checked, otherwise it is disabled.
    */
  void slotEnableRecordCheckbox(bool);

signals:
  /** This signal will be emitted whenever the user toggles the @e Minimize
    * @e To @e Panel check box. */
  void dockStatusChanged();
};

#endif

