/***************************************************************************/
/** @file     pulseplot.cpp
  * @brief    A widget that displays a pulse profile (implementation).
  *
  * @b begin      : Mon Oct 23 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <klocale.h>

#include "pulseplot.h"
#include "setiloc.h"

/*------------------------------------------------------------------------ */
PulsePlot::PulsePlot(SetiLoc* loc, QWidget *parent, const char *name )
         : SetiGraph(loc,true,false,parent,name)
{
init();
}

/*------------------------------------------------------------------------ */
PulsePlot::PulsePlot(QWidget *parent, const char *name )
         : SetiGraph(0,true,false,parent,name)
{
init();
}

/*------------------------------------------------------------------------ */
PulsePlot::~PulsePlot()
{
}

/*------------------------------------------------------------------------ */
void PulsePlot::init()
{
// Make an array twice the size of the actual pulse data. Pulse data will
// be copied twice into that array.
pulseData.resize(1024);

if(m_location)
  {
  // connect the SetiLoc signal to the new Gaussian slot of this widget.
  connect(m_location, SIGNAL(newPulse(PulseScore)),
                this, SLOT(showNewPulseData(PulseScore)));

  QString title;
  title = i18n("Location %1: Top Pulse (right-click for options)").arg(m_location->description());
  setCaption(title);
  }
setBackgroundColor(black);
resize(350, 200);
setMinimumSize(150, 100);

setTextPosition(SetiGraph::TopRight);

dataLoaded = false;

// prepare data for display
if(m_location)
  showNewPulseData(m_location->pulseScore());
}

/*------------------------------------------------------------------------ */
void PulsePlot::showNewPulseData(PulseScore score)
{
pulse = score;
pulseData.resize(2*pulse.data_len);
// convert int to double and make a copy in the upper 512 array elements
if(pulse.data_len > 0)
  {
  for(int i=0;i<pulse.data_len;i++)
    {
    pulseData[i] = (double)pulse.data[i];	
    pulseData[i+pulse.data_len] = pulseData[i];
    }
  }
setCurve(pulseData);

// generate the info text
QString it;
it.sprintf("power : %f\n"
           "mean  : %f\n"
           "period: %f\n"
           "score : %f",
           pulse.power,
           pulse.mean,
           pulse.period,
           pulse.score
           );
setInfoText(it);

dataLoaded = true;
repaint();
}

#include "pulseplot.moc"

