/***************************************************************************/
/** @file     sigmsg.cpp
  * @brief    A dialog with information about new high signals
  *           (implementation).
  *
  * @b begin      : Thu Jul 26 2001<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <stdlib.h>

#include <qlayout.h>
#include <qmemarray.h>

#include <ktextbrowser.h>

#include "sigmsg.h"
#include "gaussianplot.h"
#include "pulseplot.h"
#include "tripletplot.h"

/*------------------------------------------------------------------------ */
SigMsg::SigMsg(const SpikeScore& score, const QString& text,
               QWidget *parent, const char *name )
       : SigMsgTmpl(parent,name, false, WDestructiveClose)
{
setText(text);

QMemArray<double> d(256);
srand((unsigned int)(100000*score.power));

for(int i=0; i<256; i++)
  {
  d[i] = (double)(100.0*rand()/(RAND_MAX+1.0));
  }
int index = 16 + (int)((224.0*rand()/(RAND_MAX+1.0)));
d[index] = 20.0*(score.power*131072.0)/(8.0*score.fft_len);

SetiGraph* plot = new SetiGraph(0, false, false, this);
plot->setBackgroundColor(black);
plot->setMinimumSize(150,100);
plot->showInfoText(false);
plot->setCurve(d);
SigMsgTmplLayout->addWidget(plot, 0, 0);
}

/*------------------------------------------------------------------------ */
SigMsg::SigMsg(const GaussianScore& score, const QString& text,
               QWidget *parent, const char *name )
       : SigMsgTmpl(parent,name, false, WDestructiveClose)
{
setText(text);

GaussianPlot* plot = new GaussianPlot(this);
plot->showInfoText(false);
plot->showNewGaussianData(score);
SigMsgTmplLayout->addWidget(plot, 0, 0);
}

/*------------------------------------------------------------------------ */
SigMsg::SigMsg(const PulseScore& score, const QString& text,
               QWidget *parent, const char *name )
       : SigMsgTmpl(parent,name, false, WDestructiveClose)
{
setText(text);

PulsePlot* plot = new PulsePlot(this);
plot->showInfoText(false);
plot->showNewPulseData(score);
SigMsgTmplLayout->addWidget(plot, 0, 0);
}

/*------------------------------------------------------------------------ */
SigMsg::SigMsg(const TripletScore& score, const QString& text,
               QWidget *parent, const char *name )
       : SigMsgTmpl(parent,name, false, WDestructiveClose)
{
setText(text);

TripletPlot* plot = new TripletPlot(this);
plot->showInfoText(false);
plot->showNewTripletData(score);
SigMsgTmplLayout->addWidget(plot, 0, 0);
}

/*------------------------------------------------------------------------ */
SigMsg::~SigMsg()
{
}

/*------------------------------------------------------------------------ */
void SigMsg::setText(const QString& text)
{
QFontMetrics fm(font());

TextBox->setText(text);
TextBox->setMinimumSize( 5*fm.width("012345meww"), 8*fm.lineSpacing() );
}

#include "sigmsg.moc"
