/***************************************************************************/
/** @file     sigmsg.h
  * @brief    A dialog with information about new high signals
  *           (declaration).
  *
  * @b begin      : Thu Jul 26 2001<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef SIGMSG_H
#define SIGMSG_H

#include <qwidget.h>
#include <sigmsgtmpl.h>

#include "setidefs.h"

/***************************************************************************/
/** @file     sigmsgtmpl.h
  * @brief    The base class for the SigMsg dialog (declaration).
  *
  * Layout of the SigMsg dialog, automatically generated from the Qt
  * Designer user interface (ui) file @c sigmsgtmpl.ui. Any manual
  * modifications in this file will be overwritten.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  */

/***************************************************************************/
/** @file     sigmsgtmpl.cpp
  * @brief    The base class for the SigMsg dialog (implementation).
  *
  * Layout of the SigMsg dialog, automatically generated from the Qt
  * Designer user interface (ui) file @c sigmsgtmpl.ui. Any manual
  * modifications in this file will be overwritten.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  */

/***************************************************************************/
/** @class SigMsgTmpl
  * @brief The base class for the SigMsg dialog.
  *
  * This class is automatically generated from a Qt Designer user interface
  * (ui) file. Any manual changes to it will be overwritten.
  */

/** A dialog with information about new high signals.
  *
  * This dialog is used to report new high signals to the user.
  * It displays information about the new signals (spikes, gaussians,
  * pulses, or triplets), along with a graphical representation.
  */
class SigMsg : public SigMsgTmpl
{
Q_OBJECT

public:
  /** Creates a dialog window reporting about a new spike
    * in the current work unit.
    *
    * @param score  The SpikeScore structure containing the data
    *               of the new gaussian.
    * @param text   The text to be printed in the message box.
    * @param parent A pointer to the parent widget. If 0, this widget
    *               becomes a top-level widget.
    * @param name   The name of this widget.
    */
  SigMsg(const SpikeScore& score, const QString& text,
         QWidget* parent=0, const char* name=0);
  /** Creates a dialog window reporting about a new gaussian
    * in the current work unit.
    *
    * @overload
    *
    * @param score  The GaussianScore structure containing the data
    *               of the new gaussian.
    * @param text   The text to be printed in the message box.
    * @param parent A pointer to the parent widget. If 0, this widget
    *               becomes a top-level widget.
    * @param name   The name of this widget.
    */
  SigMsg(const GaussianScore& score, const QString& text,
         QWidget* parent=0, const char* name=0);
  /** Creates a dialog window reporting about a new pulse
    * in the current work unit.
    *
    * @overload
    *
    * @param score  The PulseScore structure containing the data
    *               of the new pulse.
    * @param text   The text to be printed in the message box.
    * @param parent A pointer to the parent widget. If 0, this widget
    *               becomes a top-level widget.
    * @param name   The name of this widget.
    */
  SigMsg(const PulseScore& score, const QString& text,
         QWidget* parent=0, const char* name=0);
  /** Creates a dialog window reporting about a new triplet
    * in the current work unit.
    *
    * @overload
    *
    * @param score  The TripletScore structure containing the data
    *               of the new triplet.
    * @param text   The text to be printed in the message box.
    * @param parent A pointer to the parent widget. If 0, this widget
    *               becomes a top-level widget.
    * @param name   The name of this widget.
    */
  SigMsg(const TripletScore& score, const QString& text,
         QWidget* parent=0, const char* name=0);
  /** The destructor.
    */
  ~SigMsg();

protected: // Protected methods
  /** Adjusts the size of the text box and prints the text.
    *
    * @param text The text to be printed.
    */
  void setText(const QString& text);
};

#endif
