/***************************************************************************/
/** @file     tripletplot.cpp
  * @brief    A widget that displays a triplet profile (implementation).
  *
  * @b begin      : Tue Oct 24 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <klocale.h>

#include <qpointarray.h>
#include <qpainter.h>

#include "tripletplot.h"
#include "setiloc.h"

/*------------------------------------------------------------------------ */
TripletPlot::TripletPlot(SetiLoc* loc, QWidget *parent, const char *name )
           : SetiGraph(loc, true, false, parent,name)
{
init();
}

/*------------------------------------------------------------------------ */
TripletPlot::TripletPlot(QWidget *parent, const char *name )
           : SetiGraph(0, true, false, parent,name)
{
init();
}

/*------------------------------------------------------------------------ */
TripletPlot::~TripletPlot()
{
}

/*------------------------------------------------------------------------ */
void TripletPlot::init()
{
// Make the array large enough.
tripletData.resize(512);

if(m_location)
  {
  // connect the SetiLoc signal to the new triplet slot of this widget.
  connect(m_location, SIGNAL(newTriplet(TripletScore)),
                this, SLOT(showNewTripletData(TripletScore)));

  QString title;
  title = i18n("Location %1: Top Triplet (right-click for options)").arg(m_location->description());
  setCaption(title);
  }
setBackgroundColor(black);
resize(350, 200);
setMinimumSize(150, 100);

//setPlotMode(SetiGraph::Lines);
setMargin(0, 0, 20, 0);

dataLoaded = false;

// prepare data for display
if(m_location)
  showNewTripletData(m_location->tripletScore());
}

/*------------------------------------------------------------------------ */
void TripletPlot::paintWidget(QPaintDevice* pd)
{
QPainter p;

SetiGraph::paintWidget(pd);
p.begin(pd);
p.translate(0, rect().bottom());

if(dataLoaded)
  {
  if(arraySize() > 0)
    {
    QBrush bru;
    bru.setStyle(SolidPattern);
    bru.setColor(Qt::green);
    QPen pen;
    pen.setColor(Qt::green);
    p.setPen(pen);
    p.setBrush(bru);

    drawPulseMarker(triplet.tpotind0_0, &p);
    drawPulseMarker(triplet.tpotind1_0, &p);
    drawPulseMarker(triplet.tpotind2_0, &p);
    }
  }

p.end();
}

/*------------------------------------------------------------------------ */
void TripletPlot::showNewTripletData(TripletScore score)
{
triplet = score;
tripletData.resize(triplet.data_len);
// convert ints to doubles
if(triplet.data_len > 0)
  {
  for(int i=0;i<triplet.data_len;i++)
    tripletData[i] = (double)triplet.data[i];
  }
setCurve(tripletData);

// generate the info text
QString it;
it.sprintf("power : %f\n"
           "mean  : %f\n"
           "period: %f\n",
           triplet.power,
           triplet.mean,
           triplet.period);
setInfoText(it);

dataLoaded = true;
repaint();
}

/*------------------------------------------------------------------------ */
void TripletPlot::drawPulseMarker(int index, QPainter* p)
{
int gw = width() - leftMargin() - rightMargin();
int x  = (int)((2*index+1.5)*gw)/arraySize();
int y  = -(int)(height() - topMargin());
int w  = gw/(2*arraySize()) + 1;
int l  = 10;

QPointArray a;
a.setPoints(3, x,y, x-w,y-l, x+w,y-l);
p->drawPolygon(a);
}

#include "tripletplot.moc"
