/***************************************************************************/
/** @file     userinfo.cpp
  * @brief    The User Info tab page and helper classes (implementation).
  *
  * @b begin      : Tue Oct 26 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <stdio.h>
#include <kapp.h>
#include <klocale.h>
#include <kdebug.h>

#include <qheader.h>
#include <qpainter.h>

#include "ksetiwatch.h"
#include "userinfo.h"
#include "setiloc.h"

/*------------------------------------------------------------------------ */
QString UserInfoListViewItem::key(int column, bool) const
{
// return value must be static
static QString ret;
QString val;
double tmp;

if(column == 2)
  {
  val = text(column);
  tmp = val.toDouble();
  val.sprintf("%f", tmp);
  ret = val.rightJustify(32, '0');
  }
else if(column == 3 || column == 4)
  {
  int colon;
  int d(0), h(0), m(0), s(0);

  val = text(column);
  colon = val.contains(':');
  if(colon == 2)
    sscanf(text(column), "%d:%d:%d", &h, &m, &s);
  else if(colon == 3)
    sscanf(text(column), "%dd:%d:%d:%d", &d, &h, &m, &s);
  tmp = 86400*d + 3600*h + 60*m + s;
  ret.sprintf("%032d", (int)tmp);
  }
else
  ret = text(column);

return(ret);
}


/*------------------------------------------------------------------------ */
UserInfo::UserInfo(QWidget* parent, const char* name) : QWidget(parent,name)
{
Ksetiwatch::TableColumn uitc[] =
  {
  {i18n("Location"), -1, AlignLeft},
  {i18n("Name"), -1, AlignLeft},
  {i18n("WU Completed"), -1, AlignRight},
  {i18n("Total CPU Time"), -1, AlignRight},
  {i18n("Average CPU Time"), -1, AlignRight},
  {i18n("Last Result"), 150, AlignRight},
  {i18n("Register Time"), 150, AlignRight}
  };

QListView_1= new popListView(this);
QListView_1->setShowSortIndicator(false);
QListView_1->setPopupStyle(popListView::ShowAnywhere);
QListView_1->move(5,1);

for(int i=0; i<(int)(sizeof(uitc)/sizeof(uitc[0])); i++)
  {
  QListView_1->addColumn(uitc[i].text,-1);
  if(uitc[i].width != -1) QListView_1->setColumnWidth(i, uitc[i].width);  	
  QListView_1->setColumnAlignment(i, uitc[i].alignment);
  }

connect(QListView_1->header(), SIGNAL(sectionClicked(int)),
                         this, SLOT(toggleSorting(int)));
sortingorder = true;

lvDict.setAutoDelete(true);

refreshList();
}

/*------------------------------------------------------------------------ */
UserInfo::~UserInfo()
{
}

/*------------------------------------------------------------------------ */
void UserInfo::resizeEvent(QResizeEvent*)
{
QListView_1->resize(this->width()-10, this->height()-5);
}

/*------------------------------------------------------------------------ */
void UserInfo::paintEvent(QPaintEvent* e)
{
QListView_1->setGridEnabled(globalopts->DrawGrid);
QWidget::paintEvent(e);
}

/*------------------------------------------------------------------------ */
void UserInfo::refreshList()
{
SetiLoc *loc;

for(loc=Ksetiwatch::locationList().first(); loc != 0;
    loc=Ksetiwatch::locationList().next())
  {
  UserInfoListViewItem* it = lvDict[loc->id()];
  if(it == 0)
    slotUpdateList(loc, Ksetiwatch::ListAdd);
  else
    refreshListItem(loc, it);
  }
}

/*------------------------------------------------------------------------ */
void UserInfo::refreshListItem(SetiLoc* loc, UserInfoListViewItem* it)
{
if(loc && it == 0)
    it = lvDict[loc->id()];
if(loc && it)
  {
  it->setText(ColLocation, loc->description());
  if(loc->exists(SetiLoc::SC_UserInfoFile))
    {
    it->setText(ColName,       loc->userName());
    it->setText(ColWuCompl,    QString::number(loc->numberOfResults()));
    it->setText(ColTotalTime,  SetiLoc::convertTime(loc->totalCPUTime(), globalopts->hms));
    it->setText(ColAvrgTime,   SetiLoc::convertTime(loc->averageCPUTime(), globalopts->hms));
    it->setText(ColLastResult, "  " + loc->lastResultTimeString());
    it->setText(ColRegTime,    "  " + loc->registerTimeString());
    }
  else
    {
    it->setText(ColName,       QString::null);
    it->setText(ColWuCompl,    QString::null);
    it->setText(ColTotalTime,  QString::null);
    it->setText(ColAvrgTime,   QString::null);
    it->setText(ColLastResult, QString::null);
    it->setText(ColRegTime,    QString::null);
    }
  }
}

/*------------------------------------------------------------------------ */
void UserInfo::slotUpdateList(SetiLoc* loc, int type)
{
UserInfoListViewItem* it;

switch(type)
  {
  case Ksetiwatch::ListAdd:
    {
    it = new UserInfoListViewItem(QListView_1);
    if(it)
      {
      lvDict.insert(loc->id(), it);
      connect(loc,  SIGNAL(updatedUserInfoData()),
              this, SLOT(slotUpdateUserInfoData()));
      refreshListItem(loc, it);
      }
    break;
    }
  case Ksetiwatch::ListEdit:
    {
    refreshListItem(loc);
    break;
    }
  case Ksetiwatch::ListDelete:
    {
    disconnect(loc,  SIGNAL(updatedUserInfoData()),
               this, SLOT(slotUpdateUserInfoData()));
    lvDict.remove(loc->id());
    break;
    }
  }
}

/*------------------------------------------------------------------------ */
void UserInfo::toggleSorting(int column)
{
sortingorder = !sortingorder;
QListView_1->setSorting(column, sortingorder);
}

/*------------------------------------------------------------------------ */
void UserInfo::slotUpdateUserInfoData()
{
SetiLoc* loc = (SetiLoc*)(sender());

if(loc)
  refreshListItem(loc);
}

#include "userinfo.moc"
